/* control.c
**
** released into the PUBLIC DOMAIN 10 jul 1994 by John Dennis
**
** The display routines for controls of various different types.
**
*/

#define DEBUG  0

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "WinSys.h"
#include "menu.h"
#include "keys.h"
#include "unused.h"

static char text[255];

/* char *MakeButton(button *b);
**
** Makes up a button.
*/

char *MakeButton(button *b, int sel)
{
    if (sel)
        sprintf(text, " %s ", b->btext);
    else
        sprintf(text, "  %s  ", b->btext);

    return text;
}


/* void ShowButton(button *b);
**
** Puts a button onto the screen, handling
** all the states of the button.
**
*/

void ShowButton(button *b)
{
    char *s;
    char *ublock = "";
    int   len;
    byte  attr;

    s   = MakeButton(b, b->select);
    len = strlen(s);
    if (b->select)
        attr = (byte)b->sattr;
    else
        attr = (byte)b->fattr;

    if (!b->down)
    {
        WWriteStr(b->x, b->y, attr, s);
        WWriteStr(b->x + len, b->y, b->battr, ublock);

        memset(text, '', sizeof(text));
        *(text + len) = '\0';

        WPutsn(b->x + 1, b->y + 1, len, b->battr, text);
    }
    else
    {
        WWriteStr(b->x + 1, b->y, attr, s);
        WWriteStr(b->x, b->y, b->battr, " ");

        memset(text, ' ', sizeof(text));
        *(text + len) = '\0';

        WPutsn(b->x+1, b->y+1, len, b->battr, text);
    }
}


/* void ShowCkbutton(ckbutton *i);
**
** Puts a Check box button onto the screen, handling
** all the states of the check box.
**
*/

void ShowCkbutton(ckbutton *i)
{
    int fa = (i->select) ? i->sattr : i->fattr;

    if (i->down)
        strcpy(text, " [x] ");
    else
        strcpy(text, " [ ] ");

    WWriteStr(i->x,  i->y, fa, text);
    WWriteStr(i->px, i->y, fa, i->prtext);
}


/* void ShowEditField(editf *i);
**
** Displays an editf control.
**
*/

void ShowEditField(editf *i)
{
    if (!i->select)
        WPutsn(i->x, i->y, i->len, i->fattr, i->buf);
    else
        WPutsn(i->x, i->y, i->len, i->sattr, i->buf);
}

void D_ShowTxt(textl *i)
{
    if (i->text)
        WWriteStr(i->x, i->y, i->fattr, i->text);
}

void D_ShowWBox(wbox *i)
{
    int  x;

    WBox(i->x1, i->y1, i->x2, i->y2, i->fattr, i->type);

    if (i->title)
    {
        x = ((i->x2 - i->x1)/2) - (strlen(i->title)/2) + i->x1;
        WWriteStr(x, i->y1, i->tattr, i->title);
    }
}


/*
**
** Please note, scroll bars are not implemented yet - this is
** preliminary code only.
**
*/


/* void D_ShowScrollBar(sbar *i);
**
** Shows the entire scrollbar.
**
*/

void D_ShowScrollBar(sbar *i, int newpos)
{
    int cnt;
    char *b[2] = { "", ""};
    char *v[2] = {"", ""};
    char  t[]  = "";
    char  s[]  = " ";

    if (newpos)
    {

        WWriteStr(i->spos+i->xy1, i->xy, i->sattr, t);
        WWriteStr(newpos+i->xy1, i->xy, i->sattr,  s);
        i->spos = newpos;
        return;
    }

    if (i->type)
    {
        WWriteStr(i->xy1, i->xy, i->battr, b[0]);
        WWriteStr(i->xy2, i->xy, i->battr, b[1]);

        for (cnt = i->xy1 + 1; cnt < i->xy2; cnt++)
            WWriteStr(cnt, i->xy, i->battr, t);

        WWriteStr(i->spos+i->xy1, i->xy, i->sattr, s);
    }
    else
    {
        WWriteStr(i->xy, i->xy1, i->battr, v[0]);
        WWriteStr(i->xy, i->xy2, i->battr, v[1]);

        for (cnt = i->xy1 + 1; cnt < i->xy2; cnt++)
            WWriteStr(i->xy, cnt, i->battr, t);

        WWriteStr(i->xy, i->spos+i->xy1, i->sattr, s);
    }
    return;
}


/*
**
** Handles messages to the scrollbar.
**
*/

int HandleScrollBar(sbar *sb, int x, int y, int lines, int pos)
{
    int wx, wy;
    int npos;
    int msg = 0;

    unused(lines);
    unused(pos);
    WndGetRel(x, y, &wx, &wy);

    if (sb->type)
    {
        npos = x - sb->xy1;
        if (npos == 0)
            msg = WM_SCRLFT;
        else
            if (npos == (sb->xy2 - sb->xy1))
                msg = WM_SCRRGT;
    }
    else
    {
        npos = y - sb->xy1;
        if (npos == 0)
            msg = WM_SCRLUP;
        else
            if (npos == (sb->xy2 - sb->xy1))
                msg = WM_SCRLDN;
    }
    if (!msg)
    {
        if (npos > sb->spos)
            msg = WM_SPGDN;
        else
            msg = WM_SPGUP;
    }
    return msg;
}

/* end of file */
