/* list.c
**
** released into the PUBLIC DOMAIN 10 jul 1994 by John Dennis
**
** Lists the messages in the messagebase.
**
*/

#include <malloc.h>
#include "msged.h"
#include "main.h"
#include "keys.h"
#include "list.h"
#include "help.h"

static MLIST *head;

/*
**
** Checks to see if the message has been selected.
**
*/

int MLselected(ulong uid)
{
    MLIST *temp;

    temp = head;
    while (temp != NULL)
    {
        if (temp->uid == uid)
            return TRUE;

        temp = temp->next;
    }
    return FALSE;
}


/*
**
** Removes a selection from the list.
**
*/

void MLkill(ulong uid)
{
    MLIST *temp, *temp2;

    /*
    ** Find it in the list, keeping track of the previous link.
    */

    temp = head;
    while (temp != NULL)
    {
        if (temp->uid == uid)
            break;

        temp2 = temp;
        temp  = temp->next;
    }
    if (temp != NULL)
    {
        /*
        ** Remove it from the list.
        */

        if (temp == head)
            head = temp->next;
        else
        {
            if (temp2)
                temp2->next = temp->next;
        }
        free(temp);
    }
}


/*
**
** Adds a new UMSGID to the list of selected messages.
**
*/

static void AddMlist(ulong uid)
{
    MLIST *temp;

    if ((temp = calloc(1, sizeof(MLIST))) == NULL)
        return;

    if (head == NULL)
        head = temp;
    else
    {
        temp->next = head;
        head       = temp;
    }
    head->uid = uid;
}


/*
**
** Frees the list of msgnumbers.
**
*/

static void KillMlist(void)
{
    MLIST *temp;

    while (head != NULL)
    {
        temp = head;
        head = head->next;
        free(temp);
    }
    head = NULL;
}

/*
**
** Re-displays the entire screen.
**
*/

static void update(MLHEAD *headers, ulong i, int y)
{
    WClear(1, y, maxx - 2, maxy - 4, cm[LS_NTXT]);
    while (i <= CurArea.messages && y <= maxy  - 4)
    {
        getheader(i, &headers[y - 1]);
        showit(&headers[y - 1], y, 0);
        i++;
        y++;
    }
}


/*
**
** Shows a header on the screen.
**
*/

static void showit(MLHEAD *h, int y, int sel)
{
    ulong msgn = (SW->showrealmsgn) ? h->umsgid : h->msgnum;
    char  line[256];
    char  msgnbuf[9];

    if (SW->showrealmsgn)
        sprintf(msgnbuf, "%c%6ld: ", (h->sel) ? '' : ' ', msgn);
    else
        sprintf(msgnbuf, "%c%4ld: ", (h->sel) ? '' : ' ', msgn);

    if (CurArea.netmail)
    {
        sprintf(line,"%s%-15.15s@%5d/%-5d to %-15.15s@%5d/%-5d RE: %-70s",
                            msgnbuf,
                            h->fr_name,
                            h->fr_net,
                            h->fr_node,
                            h->to_name,
                            h->to_net,
                            h->to_node,
                            h->subj);
    }
    else
    {
        sprintf(line,"%s%-15.15s to %-15.15s RE: %-70s",
                            msgnbuf,
                            h->fr_name,
                            h->to_name,
                            h->subj);
    }

    if (sel)
        WPutsn(1, y, 78, cm[LS_STXT], line);
    else
        if (!stricmp(h->to_name, ST->username))
            WPutsn(1, y, 78, cm[LS_ITXT], line);
        else
            WPutsn(1, y, 78, cm[LS_NTXT], line);
}


/*
**
** Gets a header from the msgbase.
**
*/

static void getheader(ulong n, MLHEAD *h)
{
    msg *i;

    memset(h, 0, sizeof(MLHEAD));
    if ((i = MsgReadHeader((uint) n, RD_HEADER)) == NULL)
        return;

    if (MLselected(i->msgnum))
        h->sel = 1;
    else
        h->sel = 0;

    h->msgnum  = n;
    h->umsgid  = i->msgnum;
    h->to_net  = i->to.net;
    h->to_node = i->to.node;
    h->fr_net  = i->from.net;
    h->fr_node = i->from.node;

    strncpy(h->subj,    i->subj,   72);
    strncpy(h->to_name, i->isto,   36);
    strncpy(h->fr_name, i->isfrom, 36);

    dispose(i);
}


/*
**
** Deletes the selected messages, or the current one if none.
**
*/

static void DeleteMsgs(ulong CurrMsgn)
{
    MLIST *temp;
    ulong  msgn;
    int    t;

    if (!confirm())
        return;

    temp              = head;
    t                 = SW->confirmations;
    SW->confirmations = 0;

    if (head == NULL)
    {
        CurArea.current = CurrMsgn;
        deletemsg();
    }
    else
    {
        while (temp != NULL)
        {
            msgn = UidToMsgn(temp->uid);
            if (msgn)
            {
                CurArea.current = msgn;
                deletemsg();
            }
            temp = temp->next;
        }
    }
    SW->confirmations = t;
}


/*
**
** Puts a list of messages up in a window on the screen.  Alows for some
** basic management of those messages.
**
*/

void  do_list(void)
{
    EVT event;
    WND *hWnd, *hCurr;
    static int in_list = 0;          /* stop recursion */
    MLHEAD *headers    = NULL;       /* headers */
    int    done        = 0;          /* finished ? */
    int    down        = 0;
    int    ForceEvt    = 0;          /* forced/piped keypress */
    int    Msg;                      /* message */
    char   temp[40];
    int    lbutton = 0;
    ulong  i, a;
    int    y;

    if (in_list || !CurArea.status)  /* stop recursion */
        return;
    else
        in_list = 1;

    /*
    ** Open the window and draw the screen and allocate the memory.
    */

    hCurr   = Wtop();
    hWnd    = WndOpen(0, 0, maxx - 1, maxy - 1, NBDR|NOSAVE, 0, cm[LS_NTXT]);
    headers = calloc(maxy, sizeof(MLHEAD));

    if (!hWnd || !headers)
    {
        if (headers)
            free(headers);

        outamemory();
    }

    WBox(0, 0, maxx - 1, maxy - 3, cm[LS_BTXT], SBDR);
    WWriteStr(3, maxy - 2, cm[LS_NTXT], "Space - Select Message     Enter - Goto message    PgUp/PgDn \033\032\030\031");
    WWriteStr(3, maxy - 1, cm[LS_NTXT], "Del   - Delete message(s)  Home  - First message   End - Last Message");

    message = KillMsg(message);
    head    = NULL;
    i       = 0;
    a       = CurArea.current;
    y       = 1;
    update(headers, a, y);

    while (!done)
    {
#if defined(MSDOS) && defined(HASCORELEFT)
        sprintf(temp, " Message Listing - [%ld] ", coreleft());
#else
        sprintf(temp, " Message Listing ");
#endif
        WPutsCen(0, cm[LS_TTXT], temp);
        showit(&headers[y - 1], y, 1);

        if (down)
        {
            /*
            ** If a selection has occured, then force cursor down.
            */

            Msg           = Key_Dwn;
            event.msg     = Msg;
            event.msgtype = WM_CHAR;
            down          = 0;
        }
        else
        {
            if (ForceEvt)
            {
                /*
                ** These events are from the mouse (no
                ** point in duplicating code).
                */

                Msg           = ForceEvt;
                event.msg     = Msg;
                event.msgtype = WM_CHAR;
                ForceEvt      = 0;
            }
            else
                Msg = MnuGetMsg(&event, hWnd->wid);
        }
        switch (event.msgtype)
        {
            case WM_MOUSE:
                switch (Msg)
                {
                    case LMOU_RPT:
                        {
                            int y1 = event.y;

                            if (y1 > maxy - 4 && lbutton)
                                ForceEvt = Key_Dwn;
                            else
                                if (y1 < 1 && lbutton)
                                    ForceEvt = Key_Up;
                        }
                        break;

                    case LMOU_CLCK:
                    case MOU_LBTDN:
                    case MOU_LBTUP:
                    case MOUSE_EVT:
                        {
                            int y1 = event.y, ok = 0;

                            if (Msg == MOU_LBTDN)
                                lbutton = 1;
                            else
                                if (Msg == MOU_LBTUP)
                                    lbutton = 0;

                            if (y1 > maxy - 4)
                            {
                                if ((Msg == MOUSE_EVT || Msg == MOU_LBTDN) && lbutton)
                                    ForceEvt = Key_Dwn;
                            }
                            else
                            {
                                if (y1 < 1)
                                {
                                    if ((Msg == MOUSE_EVT || Msg == MOU_LBTDN) && lbutton)
                                        ForceEvt = Key_Up;
                                }
                                else
                                {
                                    /*
                                    ** The event occured on the list...
                                    */

                                    if (y == y1 && (Msg == MOU_LBTUP || Msg == LMOU_CLCK))
                                    {
                                        ForceEvt = Key_Ent;
                                        continue;
                                    }

                                    showit(&headers[y - 1], y, 0);
                                    if (y > y1)
                                    {
                                        if ((a - (y - y1)) >= 1)
                                        {
                                            a -= (y - y1);
                                            ok = TRUE;
                                        }
                                    }
                                    else
                                    {
                                        if ((a + (y1 - y)) <= CurArea.messages)
                                        {
                                            a += (y1 - y);
                                            ok = TRUE;
                                        }
                                    }
                                    if (ok == TRUE)
                                    {
                                        y = y1;
                                        if (Msg == MOU_LBTUP || Msg == LMOU_CLCK)
                                            ForceEvt = Key_Ent;
                                    }
                                }
                            }
                        }
                        break;

                    default:
                        break;
                }
                break;

            case WM_CHAR:
                switch (Msg)
                {
                    case Key_PgDn:
                        i = maxy - 4 - y; /* mod 1995-04-25 PE */
                        while (i > 0 && a < CurArea.messages)
                        {
                                i--;
                                a++;
                        }
                        y = 1;
                        update(headers, a, y);
                        break;

                    case Key_PgUp:
                        if (y == 1)
                        {
                            i = maxy - 5;
                        }
                        else
                        {
                            i = y - 1;
                        }
                        while (i > 0 && a > 1)
                        {
                                i--;
                                a--;
                        }
                        y = 1;
                        update(headers, a, y);
                        break;

                    case Key_Up:
                        if (a > 1)
                        {
                            showit(&headers[y - 1], y, 0);
                            a--;
                            y--;

                            if (y < 1)
                            {
                                y = 1;
                                WScroll(1, 1, maxx - 2, maxy - 4, 0);
                                memmove((headers + 1), headers, sizeof(MLHEAD) * (maxy - 2));
                                getheader(a, &headers[0]);
                            }
                        }
                        break;

                    case Key_Dwn:
                        if (a < CurArea.messages)
                        {
                            showit(&headers[y - 1], y, 0);
                            a++;
                            y++;

                            if (y > maxy - 4)
                            {
                                y = maxy - 4;
                                WScroll(1, 1, maxx - 2, y, 1);
                                memmove(headers, (headers + 1), sizeof(MLHEAD) * (maxy - 2));
                                getheader(a, &headers[y - 1]);
                            }
                        }
                        break;

                    case Key_Home:
                        a = CurArea.first;
                        update(headers, a, y = 1);
                        break;

                    case Key_End:
                        a = CurArea.last;
                        update(headers, a, y = 1);
                        break;

                    case Key_Spc:
                        if (headers[y - 1].sel == 0)
                        {
                            headers[y - 1].sel = 1;
                            AddMlist(headers[y - 1].umsgid);
                        }
                        else
                        {
                            MLkill(headers[y - 1].umsgid);
                            headers[y - 1].sel = 0;
                        }
                        showit(&headers[y - 1], y, 1);
                        down = 1;
                        break;

                    case Key_Del:
                        DeleteMsgs(a);
                        if (a > CurArea.last)
                        {
                            a = CurArea.last;
                        }
                        update(headers, a, y = 1);
                        break;

                    case Key_A_H:
                        if (ST->helpfile)
                            DoHelp(2);
                        break;

                    case Key_Ent:
                        CurArea.current = a;
                        done            = 1;
                        break;

                    case Key_Esc:
                        done = 1;
                        break;

                    default:
                        break;
                }
                break;
        }
    }
    in_list = 0;
    KillMlist();
    free(headers);
    WClose(hWnd);
    WCurr(hCurr);
}

/* end of file */
